﻿using CommonBehaviors.Actions;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.CommonBot.Coroutines;
using Styx.Plugins;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LootBarrels
{
    public class Plugin : HBPlugin
    {
        static string lastInformationMSG;
        private static bool _lootingBarrel = false;
        private static DateTime _barrelLootingStarted;
        private static List<Barrel> _barrels = new List<Barrel>();
        private static WoWObject _currentBarrel;
        private Composite barrelHook;
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        public override string Author { get { return "SpeshulK926"; } }
        public override string Name { get { return "Loot Barrels"; } }
        public override Version Version { get { return new Version(1, 1); } }
        public override void OnEnable()
        {
            base.OnEnable();
            TreeHooks.Instance.InsertHook("InGame_Check", 0, barrelHook ?? (barrelHook = new ActionRunCoroutine(ctx => LootBarrelLoop())));
            _barrels.Clear();
            _barrels.Add(new Barrel() { ID = 251356, Name = "Cursed Queenfish" });
            _barrels.Add(new Barrel() { ID = 251387, Name = "Highmountain Salmon" });
            _barrels.Add(new Barrel() { ID = 251388, Name = "Stormray" });
            _barrels.Add(new Barrel() { ID = 251386, Name = "Mossgill Perch" });
            _barrels.Add(new Barrel() { ID = 251389, Name = "Runescale Koi" });
            _barrels.Add(new Barrel() { ID = 251390, Name = "Black Barracuda" });

            infoLog("Enabled");
        }
        public override void OnDisable()
        {
            base.OnDisable();
            TreeHooks.Instance.RemoveHook("InGame_Check", barrelHook);
            infoLog("Disabled");
        }
        private async Task<bool> LootBarrelLoop()
        {
            //infoLog("LootBarrelLoop()");

            //infoLog("Pulsing");
            var b = ObjectManager.GetObjectsOfTypeFast<WoWObject>().FirstOrDefault(f => IsEntryABarrel(f.Entry));

            //var b = ObjectManager.GetObjectsOfTypeFast<WoWObject>().FirstOrDefault(f => IsEntryABarrel(f.Entry));
            if (b != null && !_lootingBarrel)
            {
                infoLog("Looting Barrel: " + b.GetObjectName() + " : " + b.Entry);
                // we found a barrel, let's loot it.
                _lootingBarrel = true;
                _barrelLootingStarted = DateTime.Now;
                _currentBarrel = b;
            }
            else if (b == null)
            {
                infoLog("No Barrel");
                _lootingBarrel = false;
                _barrelLootingStarted = DateTime.MinValue;
                _currentBarrel = null;
            }

            if (_lootingBarrel)
            {
                //infoLog("if #1: " + (_barrelLootingStarted.Ticks < DateTime.Now.Ticks));
                // we are looting a barrel
                if (_barrelLootingStarted.Ticks < DateTime.Now.Ticks)
                {
                    // we can subtract the ticks to see how much time as passed
                    TimeSpan ts = new TimeSpan(DateTime.Now.Ticks - _barrelLootingStarted.Ticks);
                    //infoLog("ts: " + ts.TotalSeconds);
                    if (ts.TotalSeconds > 30 || b == null)
                    {
                        // more than 30 seconds has passed, so let's reset the looting.
                        _lootingBarrel = false;
                        _barrelLootingStarted = DateTime.MinValue;

                    }
                    //infoLog("not 30 seconds and b != null");
                    _currentBarrel = b;
                }
                else
                {
                    //infoLog("stop looting barrel");
                    _lootingBarrel = false;
                    _barrelLootingStarted = DateTime.MinValue;
                }
            }

            if (_lootingBarrel)
            {
                await LootBarrel();
                return true;
            }
            return false;
        }
        private async Task<bool> LootBarrel()
        {
            //infoLog("LootBarrel running");
            if (StyxWoW.IsInWorld)
            {
                if (Me.IsCasting) { SpellManager.StopCasting(); }
                //infoLog("_lootingBarrel: " + _lootingBarrel.ToString() + ", _currentBarrel != null: " + (_currentBarrel != null));

                if (_lootingBarrel && _currentBarrel != null)
                {
                    //infoLog("Moving to barrel");

                    if (_currentBarrel.Distance > 3)
                    {
                        //WoWMovement.ClickToMove(_currentBarrel.Location);
                        await CommonCoroutines.MoveTo(_currentBarrel.Location);
                    }
                    else
                    {
                        await CommonCoroutines.StopMoving();
                        _currentBarrel.Interact();
                    }

                    return true;
                }
                else
                {
                    _lootingBarrel = false;
                    _currentBarrel = null;
                }
            }

            return false;
        }
        public void infoLog(string Message, params object[] args)
        {
            if (Message == lastInformationMSG) { return; }

            Logging.Write(System.Windows.Media.Colors.LightBlue, "[LootBarrel]: {0}", Message, args);
            lastInformationMSG = Message;
        }
        bool IsEntryABarrel(uint id)
        {
            foreach (var item in _barrels)
            {
                if (item.ID == id)
                {
                    return true;
                }

            }
            return false;
        }


        public override void Pulse()
        {
            
        }
    }
    public class Barrel
    {
        public uint ID { get; set; }
        public string Name { get; set; }
    }
}
